using System;
using UnityEngine;
using UnityEngine.UI;
using UnityEngine.PostProcessing;
using System.Text;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using MVR.FileManagementSecure;
using System.Linq;
using MeshVR;
using Request = MeshVR.AssetLoader.AssetBundleFromFileRequest;
using AssetBundles;
using MVR;

namespace Feel
{
    public class OfficeWork : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel"; // edited by VamTastic (original by Frief )
        public string pluginName = "OfficeWork";
        public string pluginVersion = "1";
        public string pluginDate = "16/02" + "/2023";
        public string pluginDescription = @"Used to open and close the doors";
        #endregion

        #region Vars    

        private Atom _thisAtom;
        private FreeControllerV3 _thisControl;
        private Rigidbody camerarig = null;

        private Transform WorkPregressRoot;

        private OWEventTrigger _work_failure_event;
        private OWEventTrigger _work_end_event;
        private OWFloatTrigger _work_progress_event;

        private JSONStorableBool IsWork;

        private float _work_value = 0f;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        #endregion

        // IMPORTANT - DO NOT make custom enums. The dynamic C# complier crashes Unity when it encounters these for
        // some reason

        // IMPORTANT - DO NOT OVERRIDE Awake() as it is used internally by MVRScript - instead use Init() function which
        // is called right after creation
        public override void Init()
        {
            try
            {

                OWEventHandler.LoadAssets();

                _work_value = 0f;

                _thisAtom = containingAtom;
                _thisControl = _thisAtom.freeControllers.First(fc => fc.name == "control");

                UIDynamicButton btn1 = CreateButton("Work Failure Event", false);
                btn1.height = 80f;
                btn1.button.onClick.AddListener(OnWorkStartEventFunc);

                UIDynamicButton btn2 = CreateButton("Work End Event", false);
                btn2.height = 80f;
                btn2.button.onClick.AddListener(OnWorkEndEventFunc);

                UIDynamicButton btn3 = CreateButton("Work Progress Event", false);
                btn3.height = 80f;
                btn3.button.onClick.AddListener(OnWorkProgressEventFunc);

                IsWork = new JSONStorableBool("Is Work", false, OnIsWorking);
                RegisterBool(IsWork);

                //UIDynamicToggle toggle = CreateToggle(IsWork, false);
                //toggle.label = "IsWork";
                //toggle.height = 80;

                JSONStorableAction reset_action = new JSONStorableAction("Reset", OnWorkReset);
                RegisterAction(reset_action);

                JSONStorableAction decrease_action = new JSONStorableAction("DecreaseWork", OndecreaseWork);
                RegisterAction(decrease_action);


                JSONStorableAction support_action = new JSONStorableAction("GoToSupport", OnGoToSupport);
                RegisterAction(support_action);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Offic eWork</b></size></color>\n\n" +
                    "<color=#333>" +
                    "store the position of the target,\n" +
                    "or Track a specific target.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

                // create custom JSON storable params here if you want them to be stored with scene JSON
                // types are JSONStorableFloat, JSONStorableBool, JSONStorableString, JSONStorableStringChooser
                // JSONStorableColor


            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Start is called once before Update or FixedUpdate is called and after Init()
        void Start()
        {
            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {

            try
            {
                if (_work_progress_event != null)
                {
                    if(IsWork.val)
                    {
                        _work_value += 0.001f;
                        _work_progress_event.Trigger(_work_value);
                        if(_work_value > 1f)
                        {
                            _work_value = 1f;
                            IsWork.val = false;
                            if (_work_end_event != null) _work_end_event.Trigger();
                        }
                    }
                    
                }

                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // FixedUpdate is called with each physics simulation frame by Unity
        void FixedUpdate()
        {
            

            try
            {
                // put code in here
            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }

            
        }

        // OnDestroy is where you should put any cleanup
        // if you registered objects to supercontroller or atom, you should unregister them here
        void OnDestroy()
        {
        }

        private void OnWorkReset()
        {
            _work_value = 0f;
            IsWork.val = false;
            if(_work_progress_event != null) _work_progress_event.Trigger(_work_value);
        }

        private void OndecreaseWork()
        {
            _work_value -= 0.1f;
            if (_work_value <= 0f)
            {
                IsWork.val = false;
                _work_value = 0f;
                if (_work_failure_event != null) _work_failure_event.Trigger();
            }
            if (_work_progress_event != null) _work_progress_event.Trigger(_work_value);
        }

        private void OnGoToSupport()
        {
            Application.OpenURL("https://www.patreon.com/posts/78937142");
        }

        private void OnWorkStartEventFunc()
        {
            if (_work_failure_event == null) _work_failure_event = new OWEventTrigger(this, "Work Failure Event");
            _work_failure_event.OpenPanelActionStart();
        }

        private void OnWorkEndEventFunc()
        {
            if (_work_end_event == null) _work_end_event = new OWEventTrigger(this, "Work End Event");
            _work_end_event.OpenPanelActionStart();
        }

        private void OnWorkProgressEventFunc()
        {
            if (_work_progress_event == null) _work_progress_event = new OWFloatTrigger(this, "Work Progress Event");
            _work_progress_event.OpenPanelActionStart();
        }

        private void OnIsWorking(JSONStorableBool jb)
        {
            
        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                if (_work_failure_event != null) jc[_work_failure_event.Name] = _work_failure_event.GetJSON(base.subScenePrefix);
                if (_work_end_event != null) jc[_work_end_event.Name] = _work_end_event.GetJSON(base.subScenePrefix);
                if (_work_progress_event != null) jc[_work_progress_event.Name] = _work_progress_event.GetJSON(base.subScenePrefix);
                //SuperController.LogMessage(">> 1 GetJSON " + _movie_start_event.Name + " - " + base.subScenePrefix);
            }
            //SuperController.LogMessage(">> 1 GetJSON " + jc.ToString());
            return jc;
        }

        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true)
        {
            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);

            //_movie_start_event = new EventTrigger(this, "Movie Start Event");
            if (jc.HasKey("Work Failure Event")) _work_failure_event = new OWEventTrigger(this, "Work Failure Event");
            if (jc.HasKey("Work End Event")) _work_end_event = new OWEventTrigger(this, "Work End Event");
            if (jc.HasKey("Work Progress Event")) _work_progress_event = new OWFloatTrigger(this, "Work Progress Event");

            //SuperController.LogMessage(">> 1 RestoreFromJSON " + jc.ToString());

            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                if (jc.HasKey("Work Failure Event")) _work_failure_event.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                if (jc.HasKey("Work End Event")) _work_end_event.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
                if (jc.HasKey("Work Progress Event")) _work_progress_event.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }

    }

    public class OWEventTrigger : Trigger
    {
        public MVRScript Owner
        {
            get; private set;
        }

        private string _name;

        public string Name
        {
            get { return _name; }
        }

        public OWEventTrigger(MVRScript owner, string name)
        {
            Owner = owner;
            _name = name;
            handler = OWEventHandler.Instance;
        }

        public void OpenPanelActionStart()
        {
            if (!OWEventHandler.Loaded)
            {
                SuperController.LogError("OWEventTrigger: You need to call OWEventHandler.LoadAssets() before use.");
                return;
            }

            triggerActionsParent = Owner.UITransform;
            InitTriggerUI();
            OpenTriggerActionsPanel();

            triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = "OfficeWork";
            triggerActionsPanel.Find("Content/Tab1/Label").GetComponent<Text>().text = "Event Actions";
            triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(false);
            triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
        }

        public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
        {
            if (jc.HasKey(_name))
            {
                JSONClass tc = jc[_name].AsObject;
                if (tc != null)
                    base.RestoreFromJSON(tc, subScenePrefix, isMerge);
            }
            else if (setMissingToDefault)
            {
                base.RestoreFromJSON(new JSONClass());
            }
        }

        public void Trigger()
        {
            active = true;
            active = false;
        }
    }

    public class OWFloatTrigger : Trigger
    {
        public MVRScript Owner
        {
            get; private set;
        }

        private string _name;

        public string Name
        {
            get { return _name; }
        }

        public OWFloatTrigger(MVRScript owner, string name)
        {
            Owner = owner;
            _name = name;
            handler = OWEventHandler.Instance;
        }

        public void OpenPanelActionStart()
        {
            if (!OWEventHandler.Loaded)
            {
                SuperController.LogError("OWEventTrigger: You need to call OWEventHandler.LoadAssets() before use.");
                return;
            }

            triggerActionsParent = Owner.UITransform;
            InitTriggerUI();
            OpenTriggerActionsPanel();

            triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = "OfficeWork";
            triggerActionsPanel.Find("Content/Tab2/Label").GetComponent<Text>().text = "Value Actions";
            triggerActionsPanel.Find("Content/Tab3/Label").GetComponent<Text>().text = "Event Actions";
            triggerActionsPanel.Find("Content/Tab2").GetComponent<Toggle>().isOn = true;
            triggerActionsPanel.Find("Content/Tab1").gameObject.SetActive(false);
        }

        public void Trigger(float v)
        {
            _transitionInterpValue = Mathf.Clamp01(v);
            if (transitionInterpValueSlider != null)
                transitionInterpValueSlider.value = _transitionInterpValue;
            for (int i = 0; i < transitionActions.Count; ++i)
                transitionActions[i].TriggerInterp(_transitionInterpValue, true);
            for (int i = 0; i < discreteActionsEnd.Count; ++i)
                discreteActionsEnd[i].Trigger();
        }

        public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
        {
            if (jc.HasKey(_name))
            {
                JSONClass tc = jc[_name].AsObject;
                if (tc != null)
                    base.RestoreFromJSON(tc, subScenePrefix, isMerge);
            }
            else if (setMissingToDefault)
            {
                base.RestoreFromJSON(new JSONClass());
            }
        }

        public void Trigger(float v, List<TriggerActionDiscrete> actionsNeedingUpdateOut)
        {
            Trigger(v);
            for (int i = 0; i < discreteActionsEnd.Count; ++i)
            {
                if (discreteActionsEnd[i].timerActive)
                    actionsNeedingUpdateOut.Add(discreteActionsEnd[i]);
            }
        }
    }


    public class OWEventHandler : TriggerHandler
    {
        public static bool Loaded { get; private set; }

        private static OWEventHandler myInstance;

        private RectTransform myTriggerActionsPrefab;
        private RectTransform myTriggerActionMiniPrefab;
        private RectTransform myTriggerActionDiscretePrefab;
        private RectTransform myTriggerActionTransitionPrefab;

        public static OWEventHandler Instance
        {
            get
            {
                if (myInstance == null)
                    myInstance = new OWEventHandler();
                return myInstance;
            }
        }

        public static void LoadAssets()
        {
            SuperController.singleton.StartCoroutine(Instance.LoadAssetsInternal());
        }

        private IEnumerator LoadAssetsInternal()
        {
            foreach (var x in LoadAsset("z_ui2", "TriggerActionsPanel", p => myTriggerActionsPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionMiniPanel", p => myTriggerActionMiniPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionDiscretePanel", p => myTriggerActionDiscretePrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionTransitionPanel", p => myTriggerActionTransitionPrefab = p))
                yield return x;

            Loaded = true;
        }

        private IEnumerable LoadAsset(string assetBundleName, string assetName, Action<RectTransform> assign)
        {
            AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync(assetBundleName, assetName, typeof(GameObject));
            if (request == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null request.");
            yield return request;
            GameObject go = request.GetAsset<GameObject>();
            if (go == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null GameObject.");
            RectTransform prefab = go.GetComponent<RectTransform>();
            if (prefab == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null RectTansform.");
            assign(prefab);
        }


        void TriggerHandler.RemoveTrigger(Trigger t)
        {
            // nothing to do
        }

        void TriggerHandler.DuplicateTrigger(Trigger t)
        {
            throw new NotImplementedException();
        }

        RectTransform TriggerHandler.CreateTriggerActionsUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionsPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionMiniUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionMiniPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionDiscreteUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionDiscretePrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionTransitionUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionTransitionPrefab);
        }

        void TriggerHandler.RemoveTriggerActionUI(RectTransform rt)
        {
            UnityEngine.Object.Destroy(rt?.gameObject);
        }
    }

}